package com.example.flappybird.states; /**
 * GamePanel.java
 * Main game panel
 *
 * @author  Paul Krishnamurthy
 */

import com.example.flappybird.*;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Random;

public abstract class GameState {
	private final Random rand;

	////////////////////
	// Game variables //
	////////////////////

	// Fonts
	public static Font flappyFontBase;
	public static Font flappyFontReal;
	public static Font flappyScoreFont;
	public static Font flappyMiniFont = null;

	public static Audio audio   = new Audio();

	// Textures
	public static HashMap<String, Texture> textures = new Sprites().getGameTextures();

	protected static final int[] baseCoords = { 0, 435 };

	protected int score;           // Player score
	protected int pipeDistTracker; // Distance between pipes

	protected boolean scoreWasGreater;                // If score was greater than previous highscore

	protected Point clickedPoint = new Point(-1, -1); // Store point when player clicks
	public static ArrayList<Pipe> pipes;                   // Arraylist of Pipe objects

	protected static boolean darkTheme;                      // Boolean to show dark or light screen

	protected static Bird gameBird;

	public GamePanel gamePanel;


	public GameState(GamePanel panel) {
		this.gamePanel = panel;
		rand = new Random();

		// Try to load ttf file
		try {
			InputStream is = new BufferedInputStream(this.getClass().getResourceAsStream("/res/fonts/flappy-font.ttf"));
			flappyFontBase = Font.createFont(Font.TRUETYPE_FONT, is);

			// Header and sub-header fonts
			flappyScoreFont = flappyFontBase.deriveFont(Font.PLAIN, 50);
			flappyFontReal  = flappyFontBase.deriveFont(Font.PLAIN, 20);
			flappyMiniFont  = flappyFontBase.deriveFont(Font.PLAIN, 15);

		} catch (Exception ex) {

			// Exit is font cannot be loaded
			ex.printStackTrace();
			System.err.println("Could not load Flappy Font!");
			System.exit(-1);
		}
	}

	/**
	 * Restarts game by resetting game variables
	 */
	public void restart () {

		// Reset game statistics
		score            = 0;
		pipeDistTracker  = 0;
		scoreWasGreater  = false;

		// Get current hour with Calendar
		// If it is past noon, use the dark theme
		Calendar cal = Calendar.getInstance();
		int currentHour = cal.get(Calendar.HOUR_OF_DAY);

		// Array of bird colors
		String[] birds = new String[] {
			"yellow",
			"blue",
			"red"
		};

		// Set random scene assets
		darkTheme = currentHour > 12; // If we should use the dark theme
		// Random bird color
		String randomBird = birds[rand.nextInt(3)]; // Random bird color

		// Game bird
		gameBird = new Bird(randomBird, 172, 250, new BufferedImage[] {
			textures.get(randomBird + "Bird1").getImage(),
			textures.get(randomBird + "Bird2").getImage(),
			textures.get(randomBird + "Bird3").getImage()
		});

		// Remove old pipes
		pipes = new ArrayList<Pipe>();
	}

	/**
	 * Checks if point is in rectangle
	 * 
	 * @param      r     Rectangle
	 * @return           Boolean if point collides with rectangle
	 */
	protected boolean isTouching (Rectangle r) {
		return r.contains(clickedPoint);
	}

	public void prepareScreen(Graphics g) {
		// Set font and color
		g.setFont(flappyFontReal);
		g.setColor(Color.white);

		// Only move screen if bird is alive
		if (gameBird.isAlive()) {

			// Move base
			// Moving base effect
			int baseSpeed = 2;
			baseCoords[0] = baseCoords[0] - baseSpeed < -435 ? 435 : baseCoords[0] - baseSpeed;
			baseCoords[1] = baseCoords[1] - baseSpeed < -435 ? 435 : baseCoords[1] - baseSpeed;

		}

		// Background
		g.drawImage(darkTheme ? textures.get("background2").getImage() :
			textures.get("background1").getImage(), 0, 0, null);

		// Draw bird
		gameBird.renderBird(g);
	}

	public abstract void renderGameState(Graphics g);

	/**
	 * Needs to be called differently based on screen
	 */
	public void drawBase (Graphics g) {

		// Moving base effect
		g.drawImage(textures.get("base").getImage(), baseCoords[0], textures.get("base").getY(), null);
		g.drawImage(textures.get("base").getImage(), baseCoords[1], textures.get("base").getY(), null);

	}

	public abstract void handleKeyboardEvent(KeyEvent e);
	public abstract void handleMouseEvent(MouseEvent e);
}

