package com.example.flappybird.states;

import com.example.flappybird.FlappyBird;
import com.example.flappybird.GamePanel;
import com.example.flappybird.Highscore;
import com.example.flappybird.Pipe;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

public class PlayState extends GameState {
    protected boolean inStartGameState = false;       // To show instructions screen
    protected String medal;                           // Medal to be awarded after each game

    protected final Highscore highscore = new Highscore();

    public PlayState(GamePanel panel) {
        super(panel);
    }

    public void startGameScreen (Graphics g) {
        // Set bird's new position
        gameBird.setGameStartPos();

        // Get ready text
        g.drawImage(textures.get("getReadyText").getImage(),
                textures.get("getReadyText").getX(),
                textures.get("getReadyText").getY(), null);

        // Instructions image
        g.drawImage(textures.get("instructions").getImage(),
                textures.get("instructions").getX(),
                textures.get("instructions").getY(), null);
    }

    /**
     * Aligns and draws score using image textures
     *
     * @param mini     Boolean for drawing small or large numbers
     * @param x        X-coordinate to draw for mini numbers
     */
    public void drawScore (Graphics g, int drawNum, boolean mini, int x, int y) {

        // Char array of digits
        char[] digits = ("" + drawNum).toCharArray();

        int digitCount = digits.length;

        // Calculate width for numeric textures
        int takeUp = 0;
        for (char digit : digits) {

            // Size to add varies based on texture
            if (mini) {
                takeUp += 18;
            } else {
                takeUp += digit == '1' ? 25 : 35;
            }
        }

        // Calculate x-coordinate
        int drawScoreX = mini ? (x - takeUp) : (FlappyBird.WIDTH / 2 - takeUp / 2);

        // Draw every digit
        for (int i = 0; i < digitCount; i++) {
            g.drawImage(textures.get((mini ? "mini-score-" : "score-") + digits[i]).getImage(), drawScoreX, (mini ? y : 60), null);

            // Size to add varies based on texture
            if (mini) {
                drawScoreX += 18;
            } else {
                drawScoreX += digits[i] == '1' ? 25 : 35;
            }
        }

    }

    /**
     * Moves and repositions pipes
     */
    public void pipeHandler (Graphics g) {

        // Decrease distance between pipes
        if (gameBird.isAlive()) {
            pipeDistTracker --;
        }

        // Initialize pipes as null
        Pipe topPipe = null;
        Pipe bottomPipe = null;

        // If there is no distance,
        // a new pipe is needed
        if (pipeDistTracker < 0) {

            // Reset distance
            pipeDistTracker = Pipe.PIPE_DISTANCE;

            for (Pipe p : pipes) {

                // If pipe is out of screen
                if (p.getX() < 0) {
                    if (topPipe == null) {
                        topPipe = p;
                        topPipe.canAwardPoint = true;
                    }
                    else if (bottomPipe == null) {
                        bottomPipe = p;
                        topPipe.canAwardPoint = true;
                    }
                }
            }

            Pipe currentPipe; // New pipe object for top and bottom pipes

            // Move and handle initial creation of top and bottom pipes

            if (topPipe == null) {
                currentPipe = new Pipe("top");
                topPipe = currentPipe;
                pipes.add(topPipe);
            } else {
                topPipe.reset();
            }

            if (bottomPipe == null) {
                currentPipe = new Pipe("bottom");
                bottomPipe = currentPipe;
                pipes.add(bottomPipe);

                // Avoid doubling points when passing initial pipes
                bottomPipe.canAwardPoint = false;
            } else {
                bottomPipe.reset();
            }

            // Set y-coordinate of bottom pipe based on
            // y-coordinate of top pipe
            bottomPipe.setY(topPipe.getY() + Pipe.PIPE_SPACING);

        }

        // Move and draw each pipe

        for (Pipe p : pipes) {

            // Move the pipe
            if (gameBird.isAlive()) {
                p.move();
            }

            // Draw the top and bottom pipes
            if (p.getY() <= 0) {
                g.drawImage(textures.get("pipe-top").getImage(), p.getX(), p.getY(), null);
            } else {
                g.drawImage(textures.get("pipe-bottom").getImage(), p.getX(), p.getY(), null);
            }

            // Check if bird hits pipes
            if (gameBird.isAlive()) {
                if (p.collide(gameBird)) {
                    // Kill bird and play sound
                    gameBird.killBird();
                    audio.hit();
                } else {

                    // Checks if bird passes a pipe
                    if (gameBird.getX() >= p.getX() + p.WIDTH / 2) {

                        // Increase score and play sound
                        if (p.canAwardPoint) {
                            audio.point();
                            score ++;
                            p.canAwardPoint = false;
                        }
                    }
                }
            }
        }
    }

    public void gameOver (Graphics g) {

        // Game over text
        g.drawImage(textures.get("gameOverText").getImage(),
                textures.get("gameOverText").getX(),
                textures.get("gameOverText").getY(), null);

        // Scorecard
        g.drawImage(textures.get("scoreCard").getImage(),
                textures.get("scoreCard").getX(),
                textures.get("scoreCard").getY(), null);

        // New highscore image
        if (scoreWasGreater) {
            g.drawImage(textures.get("newHighscore").getImage(),
                    textures.get("newHighscore").getX(),
                    textures.get("newHighscore").getY(), null);
        }

        // Draw mini fonts for current and best scores
        drawScore(g, score, true, 303, 276);
        drawScore(g, highscore.bestScore(), true, 303, 330);

        // Handle highscore
        if (score > highscore.bestScore()) {

            // New best score
            scoreWasGreater = true;
            highscore.setNewBest(score); // Set in data file
        }

        // Medal
        if (score >= 10) { medal = "bronze"; }
        if (score >= 20) { medal = "silver"; }
        if (score >= 30) { medal = "gold"; }
        if (score >= 40) { medal = "platinum"; }

        // Only award a medal if they deserve it
        if (score > 9) {
            g.drawImage(textures.get(medal).getImage(),
                    textures.get(medal).getX(),
                    textures.get(medal).getY(), null);
        }

        // Buttons
        g.drawImage(textures.get("playButton").getImage(),
                textures.get("playButton").getX(),
                textures.get("playButton").getY(), null);
        g.drawImage(textures.get("leaderboard").getImage(),
                textures.get("leaderboard").getX(),
                textures.get("leaderboard").getY(), null);

    }

    @Override
    public void renderGameState(Graphics g) {
        prepareScreen(g);

        if (gameBird.isAlive()) {

            // Start at instructions state
            if (inStartGameState) {
                startGameScreen(g);

            } else {
                // Start game
                pipeHandler(g);
                gameBird.inGameBorders();
            }

            drawBase(g);                      // Draw base over pipes
            drawScore(g, score, false, 0, 0); // Draw player score

        } else {

            pipeHandler(g);
            drawBase(g);

            // Draw game over assets
            gameOver(g);
        }
    }

    @Override
    public void handleKeyboardEvent(KeyEvent e) {
        int keyCode = e.getKeyCode();
        if (gameBird.isAlive()) {
            if (keyCode == KeyEvent.VK_SPACE) {

                // Exit instructions state
                if (inStartGameState) {
                    inStartGameState = false;
                }

                // Jump and play audio even if in instructions state
                gameBird.jump();
                audio.jump();
            }
        }
    }

    @Override
    public void handleMouseEvent(MouseEvent e) {
        // Save clicked point
        clickedPoint = e.getPoint();

        if (gameBird.isAlive()) {

            // Allow jump with clicks
            if (inStartGameState) {
                inStartGameState = false;
            }

            // Jump and play sound
            gameBird.jump();
            audio.jump();

        } else {

            // On game over screen, allow restart and leaderboard buttons
            if (isTouching(textures.get("playButton").getRect())) {
                inStartGameState = true;
                gamePanel.setGameState(this);
                gameBird.setGameStartPos();
                restart();
            } else if (isTouching(textures.get("leaderboard").getRect())) {
                // Dummy message
                JOptionPane.showMessageDialog(gamePanel,
                        "We can't access the leaderboard right now!",
                        "Oops!",
                        JOptionPane.ERROR_MESSAGE);
            }

        }
    }
}
